"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var paperclip_1 = require("../native/pkg/paperclip");
var paperclip_utils_1 = require("paperclip-utils");
var mapResult = function (result) {
    if (!result) {
        return result;
    }
    if (result.Ok) {
        return result.Ok;
    }
    else {
        return { error: result.Err };
    }
};
var Engine = /** @class */ (function () {
    function Engine(_options, _onCrash) {
        var _this = this;
        if (_options === void 0) { _options = {}; }
        if (_onCrash === void 0) { _onCrash = function () { }; }
        this._options = _options;
        this._onCrash = _onCrash;
        this._listeners = [];
        this._rendered = {};
        this._loading = {};
        this._onEngineEvent = function (event) {
            if (event.kind === paperclip_utils_1.EngineEventKind.Evaluated) {
                _this._rendered[event.uri] = event;
            }
            else if (event.kind === paperclip_utils_1.EngineEventKind.Diffed) {
                var existingEvent = _this._rendered[event.uri];
                _this._rendered[event.uri] = __assign(__assign({}, existingEvent), { allDependencies: event.allDependencies, info: __assign(__assign({}, existingEvent.info), { sheet: event.sheet || existingEvent.info.sheet, preview: paperclip_utils_1.patchVirtNode(existingEvent.info.preview, event.mutations) }) });
                var addedSheets = {};
                for (var _i = 0, _a = event.allDependencies; _i < _a.length; _i++) {
                    var depUri = _a[_i];
                    // Note that we only do this if the sheet is already rendered -- engine
                    // doesn't fire an event in that scenario. So we need to notify any listener that a sheet
                    // has been added, including the actual sheet object.
                    if (!existingEvent.allDependencies.includes(depUri) &&
                        _this._rendered[depUri]) {
                        addedSheets[depUri] = _this._rendered[depUri].info.sheet;
                    }
                }
                if (Object.keys(addedSheets).length) {
                    _this._dispatch({
                        uri: event.uri,
                        kind: paperclip_utils_1.EngineEventKind.AddedSheets,
                        sheets: addedSheets,
                        allDependencies: event.allDependencies
                    });
                }
            }
        };
        this._tryCatch = function (fn) {
            try {
                return fn();
            }
            catch (e) {
                _this._onCrash();
                return null;
            }
        };
        this._dispatch = function (event) {
            // try-catch since engine will throw opaque error.
            try {
                for (var _i = 0, _a = _this._listeners; _i < _a.length; _i++) {
                    var listener = _a[_i];
                    listener(event);
                }
            }
            catch (e) {
                console.error(e);
                throw e;
            }
        };
        var io = Object.assign({
            readFile: function (uri) {
                return fs.readFileSync(new URL(uri), "utf8");
            },
            fileExists: function (uri) {
                try {
                    var url_1 = new URL(uri);
                    return fs.existsSync(url_1) && fs.lstatSync(url_1).isFile();
                }
                catch (e) {
                    console.error(e);
                    return false;
                }
            },
            resolveFile: paperclip_utils_1.resolveImportUri(fs)
        }, _options.io);
        var initNative = function () {
            _this._native = paperclip_1.NativeEngine.new(io.readFile, io.fileExists, io.resolveFile);
            // only one native listener to for buffer performance
            _this._native.add_listener(_this._dispatch);
        };
        initNative();
        this.onEvent(this._onEngineEvent);
    }
    Engine.prototype.onEvent = function (listener) {
        var _this = this;
        if (listener == null) {
            throw new Error("listener cannot be undefined");
        }
        this._listeners.push(listener);
        return function () {
            var i = _this._listeners.indexOf(listener);
            if (i !== -1) {
                _this._listeners.splice(i, 1);
            }
        };
    };
    Engine.prototype.parseFile = function (uri) {
        return mapResult(this._native.parse_file(uri));
    };
    Engine.prototype.getLoadedAst = function (uri) {
        var _this = this;
        return this._tryCatch(function () { return _this._native.get_loaded_ast(uri); });
    };
    Engine.prototype.evaluateFileStyles = function (uri) {
        var _this = this;
        return this._tryCatch(function () {
            return mapResult(_this._native.evaluate_file_styles(uri));
        });
    };
    Engine.prototype.evaluateContentStyles = function (content, uri) {
        var _this = this;
        return this._tryCatch(function () {
            return mapResult(_this._native.evaluate_content_styles(content, uri));
        });
    };
    Engine.prototype.parseContent = function (content) {
        var _this = this;
        return this._tryCatch(function () { return mapResult(_this._native.parse_content(content)); });
    };
    Engine.prototype.updateVirtualFileContent = function (uri, content) {
        var _this = this;
        this._dispatch({ kind: paperclip_utils_1.EngineEventKind.Updating, uri: uri });
        return this._tryCatch(function () {
            return mapResult(_this._native.update_virtual_file_content(uri, content));
        });
    };
    Engine.prototype._getRenderEvent = function (uri) {
        var _this = this;
        if (!this._loading[uri]) {
            this.load(uri);
        }
        if (this._rendered[uri]) {
            return Promise.resolve(this._rendered[uri]);
        }
        return new Promise(function (resolve) {
            var dispose = _this.onEvent(function (event) {
                if (event.uri === uri && event.kind === paperclip_utils_1.EngineEventKind.Evaluated) {
                    dispose();
                    resolve(event);
                }
            });
        });
    };
    Engine.prototype.getImportedSheets = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var entry, deps, depUri, event_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._getRenderEvent(uri)];
                    case 1:
                        entry = _a.sent();
                        deps = {};
                        for (depUri in this._rendered) {
                            event_1 = this._rendered[depUri];
                            if (entry.allDependencies.includes(depUri)) {
                                deps[depUri] = event_1.info.sheet;
                            }
                        }
                        return [2 /*return*/, deps];
                }
            });
        });
    };
    Engine.prototype.load = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var info, importedSheets;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._loading[uri] = true;
                        this._dispatch({ kind: paperclip_utils_1.EngineEventKind.Loading, uri: uri });
                        this._tryCatch(function () {
                            return mapResult(_this._native.load(uri, _this._options.renderPart));
                        });
                        return [4 /*yield*/, this._getRenderEvent(uri)];
                    case 1:
                        info = (_a.sent()).info;
                        return [4 /*yield*/, this.getImportedSheets(uri)];
                    case 2:
                        importedSheets = _a.sent();
                        this._dispatch({
                            kind: paperclip_utils_1.EngineEventKind.Loaded,
                            uri: uri,
                            sheet: info.sheet,
                            preview: info.preview,
                            importedSheets: importedSheets
                        });
                        return [2 /*return*/, {
                                sheet: info.sheet,
                                preview: info.preview,
                                importedSheets: importedSheets
                            }];
                }
            });
        });
    };
    return Engine;
}());
exports.Engine = Engine;
exports.evaluateAllFileStyles = function (engine, ast, resourceUrl, _loadedStyleFiles) {
    if (_loadedStyleFiles === void 0) { _loadedStyleFiles = {}; }
    var imports = paperclip_utils_1.getImports(ast);
    var map = {};
    for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
        var imp = imports_1[_i];
        var src = paperclip_utils_1.getAttributeStringValue("src", imp);
        // CSS deprecated, so don't do this.
        // if (/\.css$/.test(src)) {
        //   const cssFilePath = resolveImportFile(fs)(resourceUrl, src);
        //   const transformPath = url.fileURLToPath(cssFilePath);
        //   let importedSheetCode = _loadedStyleFiles[cssFilePath];
        //   if (!importedSheetCode) {
        //     importedSheetCode = engine.evaluateFileStyles(cssFilePath);
        //   }
        //   map[transformPath] = importedSheetCode;
        // }
    }
    return map;
};
